
/*
  SmartLock ESP8266
  Aquest projecte consisteix en una part d'un projecte de control d'accés d'un pany de porta a través d'identificadors RFID.
  Aquest Arxiu és el firmware del xip ESP8266 integrat en la placa Arduino Mega2560, i és l'encarregat de connectar Arduino
  (amb el programa Smartlock) a internet (servidor), on es faran les comprovacions d'autentificació d'accés, i retornarà els
  resultats a Smartlock. També informarà el sistema Smartlock si hi ha una connexió amb el router i si el servidor està operatiu.
  La comunicació entre la placa Arduino i el xip ESP8266 es realitzarà a través del port Serial. El xip
  ESP8266 només té un serial que és el Serial1 que correspondrà al Serial3 de la placa Arduino.
  Creat 30 de març de 2023
  per David Bassa Romera
*/

/************************************LLIBRERIES******************************************/
//Es carreguen les llibreries necessaries pel projecte
#include <ESP8266HTTPClient.h> //Llibreria que permet usar el protocol HTTP per a comunicar el controlador amb el servidor web
#include <ESP8266WiFiMulti.h> //Llibreria que permet connectar el controlador a la xarxa Wifi
#include <ESP8266WebServer.h> //Llibreria que s'utilitza per a simplificar la creació d'un servidor web utilitzant un mòdul ESP8266
#include <ESP8266Ping.h> //Llibreria que permet fer ping a un servidor web

/************************************CONSTANTS******************************************/
//Es defineixen les credencials de la connexió wifi
const char* ssid = "G55_Wifi";             //Nom de la xarxa wifi
const char* password = "tearsandrain=89";  //Contrasenya de la  xarxa Wifi
const char* host_server = "www.000webhost.com"; //Direcció del servidor


ESP8266WebServer server(80); //Es crea l'objecte server assignant el port 80

/*************************************VARIABLES*******************************************/
String webPage = ""; //Variable String on es guardarà el contingut de la pàgina principal del servidor ESP8266
String webOpen = ""; //Variable String on es guardarà el contingut de la pàgina d'obertura de la porta del servidor ESP8266
String inString; //Variable on es guardarà el text rebut pel serial
int intents; //Variable que guardarà el nombre d'intents de connexió a la xarxa wifi
char data; //Variable que guardarà cada caràcter rebut pel port serial
String targeta; //Variable que guardarà el valor de la targeta rebuda pel port Serial des d'Arduino Mega

bool online; //Variable booleana que informarà el sistema si està connectat a una xarxa wifi
bool Webserver; //Variable booleana que informarà el sistema si el servidor està operatiu


/*************************************PROGRAMA*******************************************/
//Aquesta funció només s'executa una vegada i s'encarrega de connectar ESP8266 a la xarxa wifi amb un màxim de 15 intents
void setup() {
  Serial.begin(9600);  //S'inicialitza el port serial de comunicació entre ESP8266 i Arduino Mega2560 (serial3)
  //Serial3.begin(9600);
  intents = 0; //S'inicialitza la variable intents a zero
  Webserver = true; //S'inicialitza la variable webserver a true (servidor en línia)
  targeta = ""; //S'inicialitza la variable targeta 

  WiFi.begin(ssid, password);  //Es realitza la connexió wifi passant el nom de la xarxa i la contrasenya
  Serial.print("Connectant..."); //S'escriu per serial
  
  //Es realitza la connexió wifi en arrencar el xip ESP8266
  while (WiFi.status() != WL_CONNECTED && intents < 15) { //Mentre no es detecti una connexió exitosa al wifi i hi hagi menys de 15 intents es mantindrà en el while
    delay(500); //S'atura Arduino durant 500ms
    Serial.print("."); //S'escriu pel port serial . cada 500ms
    intents++; //S'augmenta en +1 la variable intents (quan arribi a 15 sortirà del while)
  }
  //Si després de 15 intents el xip no s'ha connectat a la xarxa wifi, informarà el sistema Smartlock (port serial) que no hi ha connexió
  if (intents >=15){ 
    Serial.println("[OFFLINE]"); //S'informa el sistema Smartlock que no hi ha connexió wifi
    online = false; //S'assigna a la variable online el valor false (el sistema entra en la modalitat sense connexió)
  }else{ //Si el sistema s'ha connectat al wifi en menys de 15 intents entrarà en el if
    online = true; //S'assigna a la variable online el valor true (el sistema té connexió wifi)
    Serial.println("Connexió establerta!"); //S'escriu pel port serial
    Serial.print("IP Local: ");//S'escriu pel port serial
    Serial.println(WiFi.localIP());//S'escriu pel port serial el valor de la IP
  }

  //Es crea la pàgina principal del servidor ESP8266 afegint el contingut a la variable string webPage
  webPage += "<script>setTimeout(function(){window.location=\"https://smartlock-uoc.000webhostapp.com/index.php\"}, 100); </script>";

  //Es crea la pàgina d'obertura de porta del servidor ESP8266 afegint el contingut a la variable string webOpen
  webOpen += "<script>setTimeout(function(){window.location=\"https://smartlock-uoc.000webhostapp.com/index.php\"}, 3000); </script>";
  webOpen += "<link rel=\"stylesheet\" href=\"https://smartlock-uoc.000webhostapp.com/style.css\">";
  webOpen += "<div id=\"login\">" ;
  webOpen += "<div id=\"logo\"><img src =\"https://smartlock-uoc.000webhostapp.com/img/smartlock_logo.png\" alt=\"Smartlock\"/></div>";
  webOpen += "<div class=\"notify_door\">";
  webOpen += "<p> Porta Oberta</p></div>";
  webOpen += "</div>";

  //Si s'accedeix al servidor amb la ip carregarà la pàgina principal (webPage)
  server.on("/", []() {
    server.send(200, "text/html", webPage);//Es carrega la pàgina principal
  });

  //Si s'accedeix al servidor amb la ip/Open es carregarà la pàgina webOpen i es pasarà per serial el text [ON] (rebrà la informació Smartlock)
  server.on("/Open", []() {
    server.send(200, "text/html", webOpen); //Es carrega la pàgina WebOpen
    Serial.println("[ON]");//Escriu pel port serial
    delay(1000);//S'atura Arduino durant 1s
  });

  //Si s'accedeix al servidor amb la ip/Close carregarà la pàgina principal (webPage) i es pasarà per serial el text [OFF] (rebrà la informació Smartlock)
  server.on("/Close", []() {
    server.send(200, "text/html", webPage);//Es carrega la pàgina principal
    Serial.println("[OFF]"); //Escriu pel port serial
    delay(1000);//S'agura Arduino durant 1s
  });

  server.begin();//S'inicialitza el servidor 
  Serial.println("HTTP server started"); //S'escriu pel port serial
}

//Aquesta funció s'executa en bucle fins a l'infinit, on si no té connexió wifi, intentarà connectar-se de nou, si perd la connexió al servidor
//estarà fent comprovacions per si torna a estar operatiu, i finalment, si tot es troba online, quan rebi el valor de la targeta enviarà la informació
//al servidor per comprovar si té accés
void loop() {
  server.handleClient(); //S'inicialitza el server encarregat de rebre les peticions dels clients i llançar les funcions de callback associades
  if (Webserver = true){ //Si el servidor està operatiu comprovarà l'accés de la targeta en el servidor
    chekSerialCom(targeta); //Crida a la funció encarregada de guardar en una variable el valor d'una targeta rebuda per serial
  }
  if (WiFi.status() != WL_CONNECTED){//Si es perd la connexió wifi entrarà en el if
    reconnect (); //Crida a la funció encarregada de realitzar una connexió wifi
  }
  if (Webserver == false){ //Si el servidor està caigut entra en el if
    verify_server (); //Crida a la funció encarregada de comprovar si el servidor està operatiu (ping)
  }
}

/*************************************FUNCIONS*******************************************/
//Funció encarregada de guardar en la variable target el valor de la targeta rebuda pel port serial entre claudàtors
void chekSerialCom(String target) {
  while (Serial.available()) { //Mentre el port serial estigui operatiu entrarà en el while
    delay(25); //S'atura Arduino durant 25ms (el port seria és lent, i s'ha de donar temps per guardar el contingut de la targeta en el buffer si no queda incomplet)
    char a = Serial.read(); //Es guarda cada valor rebut pel serial en la variable a
    target.concat(String(a)); //Es guarda cada valor de la variable a en string target
    if (a == ']') { //Quan es rep ] vol dir que s'ha acabat el contingut del missatge i entra en el if
      if (target == "[OK2FACTOR]" || target == "[FAIL2FACTOR]"){
        result2factor(target);
      }else {
        check_target(target); //Es crida a la funció encarregada d'enviar el valor de la targeta al servidor
      }
      
    }
  }
}

//Funció encarregada d'enviar el valor de la targeta rebuda per paràmetre al servidor, i enviarà pel port serial la resposta rebuda per aquest
void check_target (String target){
  if(WiFi.status()== WL_CONNECTED){ //si hi ha una connexió wifi entrarà en el if
    String host = "http://smartlock-uoc.000webhostapp.com/verificacio.php"; //Es guarda en la variable host la direcció de l'arxiu php encarregat de fer la comprovació
    WiFiClient client; //Es crea l'objecte WifiClient
    HTTPClient https; //Es crea l'objecte HttpClient
    
    //S'inicialitza una connexió http entre el client i el host
    https.begin(client, host);
    
    //S'especifica el content-type header
    https.addHeader("Content-Type", "application/x-www-form-urlencoded");

    //Es guarda en la variable httpRequestData el valor rebut de la targeta (POST)
    String httpRequestData = "targeta=\"" + target +"\"";

    //Es realitza la connexió https amb el servidor enviant per post el valor de la targeta
    int httpResponseCode = https.POST(httpRequestData); //Es guarda en httpResponseCode el codi de la resposta de l'operació (si el resultat és 200 és que ha sortit correcte)
     String payload = https.getString();    //Es guarda en la variable payload el resultat del php ([ON], [OFF] o [password doble autentificació])
      Serial.print("HTTP code: "); //S'escriu pel port serial
      Serial.println(httpResponseCode);//S'escriu pel port serial el valor de la variable httpResponseCode
      Serial.println(payload);//S'escriu pel port serial el valor de la resposta del servidor ([ON], [OFF] o [password doble autentificació])
      if (httpResponseCode != 200){ //Si httpResponseCode és diferent a 200 entrarà en el if (vol dir que hi ha un error)
        Serial.println("[OFFLINE]"); //S'escriu pel port serial per informar a Smartlock que no hi ha connexió
        Webserver = false; //S'assigna a la variable Webeserver el valor false
      }
    https.end(); //Es tanca la connexió https
  }
}

//Funció que s'encarrega d'enviar el resultat de la doble autentificació al servidor perquè creï un nou registre en l'historial
void result2factor (String target){
  if(WiFi.status()== WL_CONNECTED){ //si hi ha una connexió wifi entrarà en el if
    String host = "http://smartlock-uoc.000webhostapp.com/resultat2factor.php"; //Es guarda en la variable host la direcció de l'arxiu php encarregat de guardar el resultat de la doble autentificació
    WiFiClient client; //Es crea l'objecte WifiClient
    HTTPClient https; //Es crea l'objecte HttpClient
    
    //S'inicialitza una connexió http entre el client i el host
    https.begin(client, host);
    
    //S'especifica el content-type header
    https.addHeader("Content-Type", "application/x-www-form-urlencoded");

    //Es guarda en la variable httpRequestData el valor del resultat rebut [OK2FACTOR] o [FAIL2FACTOR]
    String httpRequestData = "resultat=\"" + target +"\"";

    //Es realitza la connexió https amb el servidor enviant per post el valor del resultat
    int httpResponseCode = https.POST(httpRequestData); //Es guarda en httpResponseCode el codi de la resposta de l'operació (si el resultat és 200 és que ha sortit correcte)
      Serial.print("HTTP code: "); //S'escriu pel port serial
      Serial.println(httpResponseCode);//S'escriu pel port serial el valor de la variable httpResponseCode
      if (httpResponseCode != 200){ //Si httpResponseCode és diferent a 200 entrarà en el if (vol dir que hi ha un error)
        Serial.println("[OFFLINE]"); //S'escriu pel port serial per informar a Smartlock que no hi ha connexió
        Webserver = false; //S'assigna a la variable Webeserver el valor false
      }
    https.end(); //Es tanca la connexió https
  }
}

//Funció encarregada d'intentar fer noves connexións wifi cada 5 segons
void reconnect (){
  Serial.println("[OFFLINE]"); //S'escriu pel port serial per informar a Smartlock que no hi ha connexió wifi
  while (WiFi.status() != WL_CONNECTED) { //mentre no hi ha connexió wifi estarà en el while
    WiFi.begin(ssid, password); //S'inicia una connexió wifi amb nom de la xarxa i contrasenya
    delay(5000); //S'atura Arduino durant 5 segons
  }

  if (WiFi.status() == WL_CONNECTED){//Si es detecta que hi ha una connexió wifi entrarà en el if
    Serial.println("[ONLINE]"); //S'escriu pel port serial per informar a Smartlock que torna a haver-hi connexió wifi
    online = true; //S'assigna a la variable online el valor true 
  }
}

//Funció encarregada de verificar si el servidor està online
void verify_server (){
  while (Ping.ping(host_server) == false){//Mentre no es rebi un pong del servidor, es mantindrà en el while
    delay (5000);//S'atura Arduino durant 5s
  }
  //Si es rep un pong del servidor, vol dir que torna a estar operatiu
  Serial.println("[ONLINE]"); //S'escriu pel port serial per informar a Smartlock que el servidor està online
  Webserver = true; //S'assigna a la variable Webserver el valor true
}
