
/*
  SmartLock amb ESP8266
  Aquest projecte consisteix en un control d'accés d'un pany de porta a través d'identificadors RFID. Amb aquest codi es
  gestiona la pantalla, lector rfid, teclat, led, altaveu i pany connectats al Arduino Mega2560. També es comunicarà amb un xip
  ESP8266 a través del port serial on passarà la informació de la targeta llegida i esperarà la resposta del xip ESP8266 per donar
  accés a l'habitatge o no. Al introduïr una targeta valida l'usuari podrà accedir a l'habitatge, i si l'usuari té activada
  la doble autentificació haurà d'introduir el seu pass per poder entrar. Si el sistema no pot connectar-se al servidor
  farà la comprovació en l'àmbit local. ESP8266 informarà a través del port Serial si ha perdut connexió amb el servidor.
  Creat 30 de març de 2023
  per David Bassa Romera
*/
/************************************LLIBRERIES******************************************/
//Es carreguen les llibreries necessàries
#include <SPI.h> //S'importa la llibreria SPI que permet la comunicació pel port SPI (lector RFID)
#include <MFRC522.h> //S'importa la llibreria MFRC522 del lector RFID per poder operar amb aquest component. https://github.com/miguelbalboa/rfid?_ga=2.198023279.1756218979.1668965253-1427409422.1668965253 
#include <Servo.h> //S'importa la llibreria del ServoMotor per poder-lo controlar
#include <LiquidCrystal.h> //S'importa la llibreria de la pantalla LiquidCrystal
#include <Keypad.h> // S'importa la llibreria del keypad numeric

/************************************CONSTANTS******************************************/
//Es declaren diferents constants que guarden el número del pin dels diferents elements del sistema
//de manera que si es modifiquen aquestes connexions de la placa Arduino, només s'ha de modificar en
//les constants i no en tot el sistema

//Pins LED RGB
const int redLEDPin = 10; //S'assigna el pin del LED Vermell del LED RGB
const int greenLEDPin = 9;//S'assigna el pin del LED Verd del LED RGB
const int blueLEDPin = 11;//S'assigna el pin del LED Blau del LED RGB

//Pin altaveu
const int BuzzerPin = 13;//S'assigna el pin del Buzzer

//Pins lector RFID RC522
//Segons la llibreria, s'ha de seguir la taula de connexions dels pins a la placa, en aquest cas un Arduino MEGA https://github.com/miguelbalboa/rfid?_ga=2.198023279.1756218979.1668965253-1427409422.1668965253 
const int RFID_RST_Pin = 5; //S'assigna el pin del Reset del RFID RC522
const int RFID_SS_PIN = 53; // S'assigna el pin del SDA del RFID RC522

//Pins i posició del Servomotor
const int Servo_Open = 120; //S'assigna la posició (angle) del servo quan la porta està oberta
const int Servo_Close = 32; //S'assigna la posició (angle) del servo quan la porta està tancada
const int ServoPin = 8;//S'assigna el pin del Buzzer

//Pins, Nombre de columnes i files del teclat matriu
const byte ROWS = 4; //Es defineix el nombre de files del keypad
const byte COLS = 4; //Es defineix el nombre de columnes del Keypad
byte colPins[4] = {43,45,47,49}; //S'assigna els 4 pins de les columnes
byte rowPins[4]= {35,37,39,41}; //S'assigna els 4 pins de les files

const String code_user = "0000"; //Constant que guarda el valor del pin de doble autentificació de l'usuari


/************************************VARIABLES******************************************/
byte TarjetaActual[4]; //S'emmagatzemara la targeta llegida actual
byte Usuari1[4]= {0x65, 0xE2, 0xAA, 0x43}; //Aquesta és la targeta de l'usuari 1
byte Usuari2[4]= {0xF5, 0xE8, 0xE2, 0x52}; //Aquesta és la targeta de l'usuari 2 actualment té denegat l'accés

LiquidCrystal lcd (30,3,2,4,31,33); //Es crea l'objecte pantalla
MFRC522 mfrc522(RFID_SS_PIN, RFID_RST_Pin); //Es crea l'objecte per al lector RFID RC522

Servo myservo; //Es crea l'objecte myservo que és el motor que obrirà i tancarà la porta.

//Es crea un array amb tots els valors del teclat matriu de 4x4
char keys[ROWS][COLS] = {
{'1','2','3','A'},
{'4','5','6','B'},
{'7','8','9','C'},
{'*','0','#','D'}
};

Keypad keypad = Keypad( makeKeymap(keys), rowPins, colPins, ROWS, COLS ); //Es declara l'objecte keypad per poder utilitzar el teclat numèric

int comptador=0;            //Variable que s'utilitza per contar el nombre de tecles premudes per l'usuari (màxim 4, ja que el password és de 4 dígits, i 6 en el cas de l'administrador)
char password_1[]="1234";  //Variable amb la contrasenya de l'usuari per defecte
char key_admin_access[]="111111"; //Variable amb la clau d'accés de 6 dígits per l'accés al panell d'administració (veure valor targeta)

int intents;//Variable global que guarda el nombre d'intents de la doble autentificació (màxim 3 intents)

String inString; //Variable on es guarda el text que es rep pel port Serial3 (ESP8266)
String targeta; //Variable on es guarda el valor de la targeta llegida


/*******************************VARIABLES DE MODALITAT*************************************/
bool mode_admin = false; //Variable booleana que informa el sistema si es troba en el mode administrador o no (per poder veure el valor d'una targeta)
bool mode_control_acces = false; //Variable booleana que informa el sistema si ha detectat una targeta amb accés i está a l'espera que introdueixi la contrasenya correcta (Doble autentificació).
bool mode_nou_pass = false; //Variable booleana que informa el sistema si troba en el mode de nou_pass que s'encarrega de mostrar el valor d'una targeta
bool mode_offline = false; //Variable booleana que informa el sistema si ha perdut la connexió amb el servidor


/*************************************PROGRAMA*******************************************/
//Aquesta funció només s'executa una vegada (a l'arrencar Arduino)
void setup() {
  Serial.begin(9600);//S'inicialitza el port Serial d'Arduino 
  Serial3.begin(9600); //S'inicialitza  el port serial 3 (és el port que permet comunicar-se amb el xip ESP8266)
  SPI.begin();        //S'inicialitza  el Bus SPI
	mfrc522.PCD_Init(); //S'inicialitza  el RFID RC522
  myservo.attach(ServoPin); //S'inicialitza  el servoMotor, assignant el pin PWD amb el que es troba connectat.
  lcd.begin(16,2); //S'inicialitza la pantalla LCD amb 16 caràcters en horitzontal i dos en vertical.

  //S'informa a Arduino que els pins del LED RGB són de sortida.
  pinMode(redLEDPin, OUTPUT);
  pinMode(greenLEDPin, OUTPUT);
  pinMode(blueLEDPin, OUTPUT);

  //S'informa a Arduino que el pin del Buzzer (altaveu) és de sortida.
  pinMode(BuzzerPin, OUTPUT);

  //Es crida a la funció encarregada d'iniciar el sistema de manera que no tingui so, el LED estigui apagat, el servomotor 
  //en posició tancat i la pantalla doni un missatge de salutació
  ini();
}

//Aquesta funció s'executa en bucle fins a l'infinit i cridarà a unes funcions o d'altres en funció de la modalitat en la qual es trobi el sistema
void loop() {
  read_rfid(); //Es crida a la funció encarregada de fer la lectura de la targeta RFID

  //Si l'usuari prem la combinació de tecles definida a la variable key_admin_access llavors entrarà en el mode admin sinó estarà en el mode d'accés per RFID
  if (mode_admin == false && mode_nou_pass == false && mode_control_acces == false){ //Si el mode admin no està actiu, el mode nou pass no està activat i el control d'accés per contrasenya tampoc llavors el sistema verificarà quines tecles ha premut l'usuari
    control_admin_acces(key_admin_access); //Es crida a la funció control_admin_acces per verificar si l'usuari ha premut la combinació correcta de mode admin (6 dígits key_admin_access[])
  }
  if (mode_admin == true){//  Si el sistema es troba en mode_admin, es mostrarà el menu admin i les opcions que té l'usuari, 1=> Llegir RFID  i 2=>sortir
    menu_admin(); //Es crida a la funció encarregada de mostrar el menu_admin
  }
  if (mode_control_acces == true){//Si el sistema es troba en el mode de comprovació de doble autentificació entrarà en el if
    if (intents < 3){ //L'usuari té 3 intents per col·locar bé la contrasenya, si no el sistema haurà de tornar a llegir la targeta
      control_code(code_user);//Es crida a la funció encarregada de verificar si la clau d'accés introduïda per l'usuari es la correcta i es passa com a paràmetre la contrasenya correcte.
    }else {ini(); mode_control_acces == false;} //Si l'usuari a esgotat els 3 intents, el sistema és reseteja, surt de la modalitat doble autentificació
  }
}


/************************************FUNCIONS******************************************/
//Aquesta funció s'executa cada cop que hi ha un esdeveniment en el port serial3, es a dir, quan ESP8266 envia informació a 
//l'Arduino Mega2560
void serialEvent3() {
  while (Serial3.available()) { //Es quedarà dintre del bucle mentre el port Serial3 estigui actiu
    char inChar = Serial3.read();//Lectura de dades del Serial3
    Serial.write(inChar);// S'escriu les dades rebudes pel port serial (inChar) en el Serial1
    inString += inChar; //Es guarden tots els valor rebuts pel Serial 3 un a un
    // Es busca si hi ha el text [ON], [OFF], [OFFLINE], [ONLINE] o un text entre claudàtors [] (contrasenya TwoFactor)
    if (inChar == ']') {
      if (inString.indexOf("[ON]")>0) { //Si el valor rebut pel serial és [ON] entra en el if
        acces_correct(); //Es crida a la funció encarregada d'obrir la porta
      }
      else if (inString.indexOf("[OFF]")>0) {//Si el valor rebut pel serial és [OFF] entra en el if
        acces_denied(); //Es crida a la funció encarregada d'informar a l'usuari que no té accés
      }
      else if (inString.indexOf("[OFFLINE]")>0) {//Si el valor rebut pel serial és [OFFLINE] entra en el if
        mode_offline = true; //Es posa la constant mode_offline en true per notificar al sistema que s'ha perdut la connexió i funcionarà a nivell local
      }
      else if (inString.indexOf("[ONLINE]")>0) {//Si el valor rebut pel serial és [ONLINE] entra en el if
        mode_offline = false;//Es posa la constant mode_offline en false per notificar al sistema que hi ha connexió i funcionarà en remot (servidor)
      }else {
        String code; //Es declara la variable on es guardarà el codi rebut pel port Serial3 (ESP8266) del servidor
        code = inString.substring(inString.length()-5, inString.length()-1); //Es guarden els quatre dígits eliminant els claudàtors d'inici i final
        code_user = code; //S'assigna el valor de la variable code a la variable global code_user
        mode_control_acces = true; //Es col·loca el sistema en la modalitat de doble autentificació
        lcd.clear();//S'esborra el contingut de text de la pantalla
        lcd.print("Contrasenya?");  //S'escriu text per pantalla
      }
      inString = ""; //Es reseteja el valor de inString
    }
  }
}

//Aquesta funció s'encarrega de fer una sèrie d'accions si l'usuari té accés a l'habitatge
//Posar el LED en color verd, fer sonar l'altaveu, moure el servomotor en posició oberta
//enviar un text a la pantalla, etc.
void acces_correct(){
  analogWrite (redLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED vermell
  analogWrite (greenLEDPin, 255);//Es posa un valor de sortida 255 (encès) al pin del LED verd
  analogWrite (blueLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED Blau
  delay(100); //Es para Arduino durant 100ms
  tone(BuzzerPin, 2000, 1000); //S'emet un altre so més greu per donar a entendre que s'accepta l'accès
  myservo.write(Servo_Open); //Es col·loca el servoMotor en posició oberta (porta oberta)
  lcd.setCursor(0,1); //Es situa el cursor de la pantalla en la segona fila de la pantalla
  lcd.print("Acces Concedit"); //Es mostra el text d'accés concedit per la pantalla
  delay(3000);//Es para Arduino durant 3000ms
  noTone(BuzzerPin); //S'apaga el so del so del buzzer
  lcd.clear(); //S'esborra tot el contingut de text de la pantalla
  if (mode_offline == false){ //Si el sistema està en mode online entra en el if
    lcd.print("Hola!"); //S'escriu un text per la pantalla
  }else { //Si el sistema està en el mode offline entra en el if
    lcd.print("Hola! - OFFLINE"); //S'escriu un text per la pantalla
  }
  myservo.write(Servo_Close);//Es col·loca el servoMotor en posició tancat (tanca la porta)
  analogWrite (greenLEDPin, 0);//S'apaga el LED verd (valor sortida 0).
  mode_control_acces = false; //S'assigna el valor de mode_acontrol_acces en false perquè l'usuari ja ha entrat a l'habitatge
}

//Aquesta funció s'encarrega de fer una sèrie d'accions si l'usuari no té accés al sistema
//Posar el LED en color vermell, fer sonar l'altaveu, posar el Servomotor en posició tancat,
//i enviar un text per pantalla.
void acces_denied(){
  analogWrite (redLEDPin, 255);//Es posa un valor de sortida 255 (encès) al pin del LED vermell
  analogWrite (greenLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED verd
  analogWrite (blueLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED blau
  myservo.write(Servo_Close);//Es col·loca el servoMotor en posició tancat (per assegurar que està tancada la porta)
  delay(100); //S'atura arduino durant 100ms
  analogWrite (redLEDPin, 0); // S'apaga el LED de color vermell (valor sortida 0)
  lcd.setCursor(0,1); //Es situa el cursor de la pantalla en la segona fila de la pantalla
  lcd.print("Acces Denegat");//Es mostra el text d'accés denegat per pantalla
  tone(BuzzerPin, 1000, 200); //S'emet un altre so més agut per donar a entendre que es denega l'accés
  delay(1000); //S'atura Arduino durant 1000ms
  noTone(BuzzerPin); //S'apaga el Buzzer (no emet so)
  delay(3000);//S'atura Arduino durant 3000ms
  lcd.clear();//S'esborra tot el contingut de text de la pantalla
  if (mode_offline == false){//Si el sistema està en mode online entra en el if
    lcd.print("Hola!"); //S'escriu un text per la pantalla
  }else {//Si el sistema està en el mode offline entra en el if
    lcd.print("Hola! - OFFLINE"); //S'escriu un text per la pantalla
  }
  mode_control_acces = false; //S'assigna el valor de mode_acontrol_acces en false perque l'usuari no té accés
}

//Aquesta funció s'encarrega de fer una sèrie d'accions si l'usuari ha introduït malament la contrasenya de la doble
//autentificació com posar el LED en color vermell, fer sonar l'altaveu, posar el Servomotor en posició tancat,
//i enviar un text per pantalla.
void acces_denied_intent(){
  analogWrite (redLEDPin, 255);//Es posa un valor de sortida 255 (encès) al pin del LED vermell
  analogWrite (greenLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED verd
  analogWrite (blueLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED blau
  myservo.write(Servo_Close);//Es col·loca el servoMotor en posició tancat (per assegurar que està tancada la porta)
  delay(100); //S'atura Arduino durant 100ms
  analogWrite (redLEDPin, 0); // S'apaga el LED de color vermell (valor sortida 0)
  lcd.setCursor(0,1); //Es situa el cursor de la pantalla en la segona fila de la pantalla
  lcd.print("Acces Denegat");//Es mostra el text d'accés denegat per pantalla
  tone(BuzzerPin, 1000, 200); //S'emet un altre so més agut per donar a entendre que es denega l'accés
  delay(1000); //S'atura Arduino durant 1000ms
  noTone(BuzzerPin); //S'apaga el Boozer (no emet so)
  delay(3000);//S'atura Arduino durant 3000ms
  lcd.clear();//S'esborra tot el contingut de text de la pantalla
  lcd.print("Contrasenya?");//S'escriu un text per la pantalla
}

//Aquesta funció s'encarrega de preparar el sistema quan arrenca, on no tindrà so, el LED RGB estarà apagat, 
//el servomotor en posició tancat i la pantalla mostra un missatge
void ini(){
  analogWrite (redLEDPin, 0); //Es posa un valor de sortida 0 (apagat) al pin del LED vermell
  analogWrite (greenLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED verd
  analogWrite (blueLEDPin, 0);//Es posa un valor de sortida 0 (apagat) al pin del LED blau
  noTone(BuzzerPin);//S'apaga el buzzer perquè no emeti cap so
  myservo.write(Servo_Close); //Servomotor en posició tancat, la porta per defecte estarà tancada
  lcd.clear();//S'esborra tot el contingut de text de la pantalla
  if (mode_offline == false){//Si el sistema està en mode online entra en el if
    lcd.print("Hola!"); //S'escriu un text per la pantalla
  }else {//Si el sistema està en el mode offline entra en el if
    lcd.print("Hola! - OFFLINE"); //S'escriu un text per la pantalla
  }
  intents = 0; //Sinicialitza la variable intents a zero
}

//Aquesta funció s'encarrega de llegir les targetes del lector RFID, si llegeix una tarjeta emet
//un so confirmant la lectura, i s'emmagatzema en la variable TarjetaActual el valor de la tarjeta
void read_rfid(){
  	if ( mfrc522.PICC_IsNewCardPresent()) //Si el lector RFID llegeix una targeta retorna un true
        {  
          tone(BuzzerPin, 3000, 200); //Al llegir una targeta s'emet un so confirmant la lectura correcte
            if ( mfrc522.PICC_ReadCardSerial())  //Es selecciona una targeta
            {
                  targeta = "["; //S'assigna a la variable targeta el valor [ a l'inici
                  for (byte i = 0; i < mfrc522.uid.size; i++) {//Es recorren tots els bytes de la tarjeta
                      TarjetaActual[i]=mfrc522.uid.uidByte[i];//Es guarda el valor de la targeta en la variable TarjetaActual
                      //Es guarden els valors de la targeta llegida en un String
                      targeta.concat(String(mfrc522.uid.uidByte[i] < 0x10 ? " 0" : " "));
                      targeta.concat(String(mfrc522.uid.uidByte[i], HEX));
                  }
                  targeta.concat(String("]")); //Al final de l'string s'afegeix  un ]
                  mfrc522.PICC_HaltA();  // Es tanca  la lectura de la targeta
                  if (mode_admin == false && mode_nou_pass == false && mode_offline == false){ //Si el sistema no està en mode administrador i té connexió amb el servidor llavors enviarà el valor de la targeta pel serial3 (xip ESP8266)
                    Serial3.println(targeta); //S'escriu pel port Serial 3 el valor de targeta (es passa el valor de la targeta al xip ESP8266)
                  }
                  if (mode_admin == false && mode_nou_pass == true){ //Si el sistema està en mode admin i nou pass (nova targeta)
                     new_pass (targeta); //Crida a la funció new_pass encarregada de mostrar el valor de la targeta per pantalla
                  }
                  if (mode_admin == false && mode_nou_pass == false && mode_offline == true){ //Si el sistema no està en mode administrador i no té connexió amb el servidor, farà la comprovació en local
                    acces_control(); //crida a la funció encarregada de verificar l'accés de la targeta a nivell local
                  }  
            }
	}
}

//Aquesta funció s'encarregarà de comparar dues targetes (la targeta llegida amb la targeta emmagatzemada)
//si són iguals retornarà true i si no falç
boolean compareArray(byte array1[],byte array2[])
{
  //Es van comparant els dos arrays que rep per funció, si algun byte es diferent retorna false
  if(array1[0] != array2[0])return(false);
  if(array1[1] != array2[1])return(false);
  if(array1[2] != array2[2])return(false);
  if(array1[3] != array2[3])return(false);
  return(true); // Si tots els bytes son iguals retornarà un true
}

//Aquesta funció s'encarrega de comprovar les credencials de la targeta, si té permisos d'accés llavors posa Smartlock en la modalitat de doble autentificació
//si introdueix la contrasenya correcta donarà accés al sistema, per al contrari si no en té, crida a la funció acces denied
void acces_control(){
  if(compareArray(TarjetaActual,Usuari1)){//Es crida a la funció encarregada de verificar si la targeta està al sistema, si existeix llavors retornarà true i entrarà en el if
    lcd.clear(); //S'esborra tot el contingut de la pantalla
    lcd.print("Codi acces?");//Es mostra un text per pantalla demanant la clau d'accés
    mode_control_acces =true;//Es posa el sistema en mode_control_acces, que és l'encarregat de comprobar si la clau d'accés introduïda és correcte
    code_user = "0000"; //S'assigna el valor per defecte de la contrasenya de doble autentificació a 0000
  }
  else{//Si la targeta no està registrada al sistema entra en else
    acces_denied(); //Es crida a la funció acces_denied
  }
}

//Aquesta funció s'encarrega de verificar si la contrasenya que introdueix l'usuari és correcte o no, si és correcte obre el pany intel·ligent i
//si no denega l'accés.
void control_code(String password){
  char key = keypad.getKey(); //Variable on es guarda la tecla premuda
  char clau[4];            //Variable on es guarden els 4 caràcters seguits que ha premut l'usuari
  if (key != NO_KEY){     //Si es detecta que l'usuari ha premut alguna tecla, entra en el if
    clau[comptador]=key;          // Es guarda caràcter a caràcter cada cop que es toca una tecla en la variable clau[]
    lcd.setCursor(comptador,1); //Se situa el cursor de la pantalla en la segona fila i en la posició igual al comptador
    lcd.print("*"); //S'escriu per la pantalla un * cada cop que l'usuari prem una tecla
    comptador=comptador+1;              //S'augmenta la variable comptador per guardar cada caràcter en una nova posició fins arribar a 4 tecles guardades
    if(comptador==4)  {//Si l'usuari ja ha premut 4 tecles seguides es verificarà si la clau es correcte
      String claut_str = String (clau); //Es converteix a string i es guarda el valor de clau en la variable claut_str
      claut_str = claut_str.substring(0, 4); //S'assegura que en la variable clau_str només tindrà els 4 primers caràcters
      if(claut_str.compareTo(password) == 0) //Es comparen les dues cadenes (password introduir per usuari vs password correcte) si el valor és 0 vol dir que són iguals
      {
        acces_correct(); //Es crida a la funció acces_correct
        comptador=0;// Es reseteja la variable a 0 per poder tornar a guardar 4 tecles
        intents = 0;
        Serial3.println("[OK2FACTOR]"); //S'escriu pel port Serial 3 per informar el servidor que l'usuari a introduït la contrasenya correctament
      } else {acces_denied_intent();intents++; } //Es denega l'accés al sistema si la clau no és correcte
      comptador=0;  //Es reseteja la variable a 0 per poder tornar a guardar 4 tecles
    }
  }
  if (intents >=3){ //Si l'usuari ha realitzat més de tres intents erronis s'informarà a l'historial del servidor
    Serial3.println("[FAIL2FACTOR]"); //S'informa a l'ESP8266 que l'usuari ha introduït malament la contrasenya
  }
}

//Aquesta funció s'encarrega de verificar si l'usuari a introduït la clau d'accés d'administrador (6 dígits)
void control_admin_acces(char password[]){
  char key = keypad.getKey(); //Variable on es guarda la tecla premuda
  char clau[6];            //Variable on es guarden els 6 caràcters seguits que s'han premut per l'usuari
  if (key != NO_KEY){     //Si es detecta que l'usuari ha premut alguna tecla entra en el if
    clau[comptador]=key;          // Es guarda caràcter a caràcter cada cop que es toca una tecla en la variable clau[]
    comptador=comptador+1;              //S'augmenta la variable comptador per guardar cada caràcter en una nova posició fins arribar a 6 tecles guardades
    if(comptador==6)  {//Si l'usuari ja ha premut 6 tecles seguides es verificarà si la clau és correcte
      if(clau[0]==password[0]&&clau[1]==password[1]&&clau[2]==password[2]&&clau[3]==password[3]&&clau[4]==password[4]&&clau[5]==password[5]) //Es verifica caràcter a caràcter si la variable clau (introduïda per l'usuari) és igual a la variable on es guarda la clau d'accés de l'administrador (key_admin_access)
      {
        lcd.clear();//S'esborra tot el contingut de text de la pantalla
        lcd.print("Llegir RFID?");//S'escriu un text per pantalla
        lcd.setCursor(0,1); //Es situa el cursor de la pantalla en la segona fila
        lcd.print("1 =>Si   2=>No");//Es mostra per pantalla les dues opcions disponibles de l'usuari.
        comptador=0;//Es reseteja la variable a 0 per poder tornar a guardar 6 tecles
        mode_admin=true;//Es posa la variable mode_admin en True, per fer que el sistema entri en el mode d'administrador
        tone(BuzzerPin, 1000, 200); //S'emetrà un altre só greu per donar a entendre que el password és correcte
        delay(1000); //S'atura Arduino durant 1000ms
        noTone(BuzzerPin); //S'apaga el so de l'altaveu
        menu_admin();//Es crida a la funció menu_admin encarregada de mostrar el panell de lectura de targeta
      }
      comptador=0;  //Es reseteja la variable a 0 per poder tornar a guardar 4 tecles
    }
  }
}

//Aquesta funció s'encarrega de mostrar el menú d'administrador per veure el valor d'una targeta
void menu_admin(){
  char key = keypad.getKey(); //Variable on es guarda la tecla premuda
  if (key != NO_KEY){//Es verifica si es detecta que l'usuari ha premut alguna tecla, si és així entre en el if
    if (key == '1'){ //Si l'usuari ha premut l'opció 1 podrà veure el valor de la targeta
      lcd.clear();//S'esborra el contingut de text de la pantalla
      lcd.print("Valor RFID");  //S'escriu text per pantalla
      mode_nou_pass = true; //S'assigna a la variable mode_nou_pass valor true per poder llegir el valor d'una targeta
      mode_admin = false; //S'assigna el valor false al mode_admin (surt del panell d'administració)
    }if (key == '2') {//Si l'usuari ha premut l'opció 2 sortirà del mode administrador
      lcd.clear();//S'esborra tot el contingut de la pantalla
      lcd.print("Hola!");  //S'escriu text per pantalla
      mode_admin = false; //S'assigna valor false al mode_admin (surt del panell d'administració)
    }
  }
}

//Aquesta funció permet visualitzar el valor d'una targeta per pantalla, així l'usuari podrà saber el valor d'una targeta
//per donar-la d'alta al sistema. Rebrà el valor de la targeta pels paràmetres
void new_pass (String targeta){
      mode_admin = false; //S'assigna a la variable mode_admin el valor false, ja que es surt del mode administrador
      mode_nou_pass = false; //S'assigna a la variable mode_nou_pass el valor false, ja que es surt del mode new pass
      lcd.setCursor(0,1); //Es situa el cursor de la pantalla en la segona fila
      lcd.print(targeta);  //S'escriu el text amb el valor de la targeta
      delay (5000); //S'atura Arduino durant 5 segons perquè l'usuari pugui apuntar el valor
      ini();//Es crida a la funció ini() encarragada d'iniciar el sistema
}

